#ifndef SL_COMPLEX_CARITH_H
#define SL_COMPLEX_CARITH_H

/*
 ******************************
 * Scientific Library (GNU Public Licence)
 *
 * Author: Laurent Deniau, Laurent.Deniau@cern.ch
 *
 * $Id: carith.h,v 1.6 1998/11/05 13:03:55 paubert Exp $
 *
 * Suggestions: sl@mathinsa.insa-lyon.fr
 * Bugs:   sl-bugs@mathinsa.insa-lyon.fr
 *
 * For more information, please see the sl++ Home Page:
 * http://wwwinfo.cern.ch/~ldeniau/sl.html
 *
 ******************************
 */

#ifndef SL_COMPLEX_H
#error <sl/complex/carith.h> must be included via <sl/complex.h>
#endif

/* 
 * Promotion for scalar parameters will be used when the compiler will
 * be compatible with C++ ANSI
 */

#ifdef HAVE_NAMESPACE
namespace sl
{
#endif    

// Plus

template <typename T>
inline Complex<T> const&
operator + (Complex<T> const& z)
{
  return z;
}

// Minus

template <typename T>
inline Complex<T>
operator - (Complex<T> const& z) 
{
  return Complex<T> (-re(z), -im(z));
}

// Addition

template <typename T1, typename T2>
inline typename promote<Complex<T1>,Complex<T2> >::promoted_t
operator + (Complex<T1> const& z1, Complex<T2> const& z2)
{
  typedef typename promote<Complex<T1>,Complex<T2> >::promoted_t promoted_t;
  return promoted_t (re(z1)+re(z2), im(z1)+im(z2));
}

template <typename T>
inline typename promote<Complex<T>,T>::promoted_t
operator + (Complex<T> const& z, T const x)
{
  typedef typename promote<Complex<T>,T>::promoted_t promoted_t;
  return promoted_t (re(z)+x, im(z));
}

template <typename T>
inline typename promote<T,Complex<T> >::promoted_t
operator + (T const x, Complex<T> const& z)
{
  typedef typename promote<T,Complex<T> >::promoted_t promoted_t;
  return promoted_t (re(z)+x, im(z));
}

// Subtraction

template <typename T1, typename T2>
inline typename promote<Complex<T1>,Complex<T2> >::promoted_t
operator - (Complex<T1> const& z1, Complex<T2> const& z2)
{
  typedef typename promote<Complex<T1>,Complex<T2> >::promoted_t promoted_t;
  return promoted_t (re(z1)-re(z2), im(z1)-im(z2));
}

template <typename T>
inline typename promote<Complex<T>,T>::promoted_t
operator - (Complex<T> const& z, T const x) 
{
  typedef typename promote<Complex<T>,T>::promoted_t promoted_t;
  return promoted_t (re(z)-x, im(z));
}

template <typename T>
inline typename promote<T,Complex<T> >::promoted_t
operator - (T const x, Complex<T> const& z) 
{
  typedef typename promote<T,Complex<T> >::promoted_t promoted_t;
  return promoted_t (x-re(z), -im(z));
}

// Multiplication

template <typename T1, typename T2>
inline typename promote<Complex<T1>,Complex<T2> >::promoted_t
operator * (Complex<T1> const& z1, Complex<T2> const& z2)
{
  typedef typename promote<Complex<T1>,Complex<T2> >::promoted_t promoted_t;
  return promoted_t (re(z1)*re(z2)-im(z1)*im(z2), 
                    re(z1)*im(z2)+im(z1)*re(z2));
}

template <typename T>
inline typename promote<Complex<T>,T>::promoted_t
operator * (Complex<T> const& z, T const x) 
{
  typedef typename promote<Complex<T>,T>::promoted_t promoted_t;
  return promoted_t (re(z)*x, im(z)*x);
}

template <typename T>
inline typename promote<T,Complex<T> >::promoted_t
operator * (T const x, Complex<T> const& z) 
{
  typedef typename promote<T,Complex<T> >::promoted_t promoted_t;
  return promoted_t (x*re(z), x*im(z));
}

// Division

template <typename T1, typename T2>
inline typename promote<Complex<T1>,Complex<T2> >::promoted_t
operator / (Complex<T1> const& z1, Complex<T2> const& z2)
{
//  typedef typename promote<T1,T2>::promoted_t promoted_et;
  typedef typename promote<Complex<T1>,Complex<T2> >::promoted_t promoted_t;

//  promoted_et nr, ni;
  double nr, ni;

  if (abs(re(z2)) <= abs(im(z2))) {
//    promoted_et t = promoted_et(re(z2))/im(z2);
//    promoted_et n = im(z2)*(1+t*t);
    double t = (re(z2))/im(z2);
    double n = im(z2)*(1+t*t);
    nr = (re(z1)*t+im(z1))/n;
    ni = (im(z1)*t-re(z1))/n;
  } else {
//    promoted_t t = promoted_t(im(z2))/re(z2);
//    promoted_t n = re(z2)*(1+t*t);

     double t = (im(z2))/re(z2);
     double n = re(z2)*(1+t*t);
     nr = (re(z1)+im(z1)*t)/n;
     ni = (im(z1)-re(z1)*t)/n;
  }
  return promoted_t (nr, ni);
}

template <typename T>
inline typename promote<Complex<T>,T>::promoted_t
operator / (Complex<T> const& z, T const x) 
{
  typedef typename promote<Complex<T>,T>::promoted_t promoted_t;
  return promoted_t (re(z)/x, im(z)/x);
}

template <typename T>
inline typename promote<T,Complex<T> >::promoted_t
operator / (T const x, Complex<T> const& z) 
{
//  typedef typename promote<T,T>::promoted_t promoted_et;
  typedef typename promote<T,Complex<T> >::promoted_t promoted_t;

 // promoted_et nr, ni;
  double nr, ni;

  if (abs(re(z)) <= abs(im(z))) {
 //   promoted_et t = promoted_et(re(z))/im(z);
 //   promoted_et n = im(z)*(1+t*t);
    double t = (re(z))/im(z);
    double n = im(z)*(1+t*t);

    nr = x*t/n;
    ni = -x/n;
  } else {
//    promoted_et t = promoted_et(im(z))/re(z);
//    promoted_et n = re(z)*(1+t*t);
    double t = (im(z))/re(z);
    double n = re(z)*(1+t*t);
    nr = x/n;
    ni = -x*t/n;
  }

  return promoted_t (nr, ni);
}

#ifdef HAVE_NAMESPACE
}
#endif

#endif
// SL_COMPLEX_CARITH_H
