#ifndef SL_COMPLEX_CIOOPS_H
#define SL_COMPLEX_CIOOPS_H

/*
 ******************************
 * Scientific Library (GNU Public Licence)
 *
 * Author: Laurent Deniau, Laurent.Deniau@cern.ch
 *
 * $Id: cioops.h,v 1.4 1998/06/25 13:48:34 paubert Exp $
 *
 * Suggestions: sl@mathinsa.insa-lyon.fr
 * Bugs:   sl-bugs@mathinsa.insa-lyon.fr
 *
 * For more information, please see the sl++ Home Page:
 * http://wwwinfo.cern.ch/~ldeniau/sl.html
 *
 ******************************
 */

#ifndef SL_COMPLEX_H
#error <sl/complex/cioops.h> must be included via <sl/complex.h>
#endif

#ifdef HAVE_NAMESPACE
namespace sl 
{
#endif




// I/O operators

template <typename T> 
ostream&
operator << (ostream& os, Complex<T> const& z)
{
  Char lc, mc, rc;

  ComplexFormat::get(lc,mc,rc);

  if (lc) {
    return os << lc << z.real() << mc << z.imag() << rc;
  } else {
    if (z.real() >= 0) os << ' ';
    if (z.imag() >= 0) 
      return os << z.real() << " + " <<  z.imag() << rc;
    else
      return os << z.real() << " - " << -z.imag() << rc;
  }
}

template <typename T>
istream&
operator >> (istream& is, Complex<T>& z)
    {
        T r, i = 0;
        Char lc, mc, rc, c = 0;

        ComplexFormat::get(lc,mc,rc);

        if (lc) {
            is >> c;
            if (c == lc) {
                is >> r >> c;
                if (c == mc) is >> i >> c;
                if (c != rc) is.clear(ios::badbit);
            } else {
                is.putback(c);
                is >> r;
            }
        } else {
            is >> r >> c;
            switch(c) {
            case '+':
                is >> i >> c;
                if (c != rc) is.clear(ios::badbit);
                break;
            case '-':
                is >> i >> c; i = -i;
                if (c != rc) is.clear(ios::badbit);
                break;
            default: is.putback(c);
            }
        }
        if (is) z = Complex<T> (r, i);
        return is;
    }

#ifdef HAVE_NAMESPACE
}
#endif

#endif // SL_COMPLEX_CIOOPS_H
