#ifndef SL_COMPLEX_CMISC_H
#define SL_COMPLEX_CMISC_H

/*
 ******************************
 * Scientific Library (GNU Public Licence)
 *
 * Author: Laurent Deniau, Laurent.Deniau@cern.ch
 *
 * $Id: cmisc.h,v 1.5 1998/07/08 13:25:36 paubert Exp $
 *
 * Suggestions: sl@mathinsa.insa-lyon.fr
 * Bugs:   sl-bugs@mathinsa.insa-lyon.fr
 *
 * For more information, please see the sl++ Home Page:
 * http://wwwinfo.cern.ch/~ldeniau/sl.html
 *
 ******************************
 */

#ifndef SL_COMPLEX_H
#error <sl/complex/cmisc.h> must be included via <sl/complex.h>
#endif

#ifdef HAVE_NAMESPACE
namespace sl 
{
#endif

template <typename T>
inline T
re (Complex<T> const& z)
{
  return z.real();
}

template <typename T>
inline T
im (Complex<T> const& z)
{
  return z.imag();
}

template <typename T>
inline T
real (Complex<T> const& z)
{
  return z.real();
}

template <typename T>
inline T
imag (Complex<T> const& z)
{
  return z.imag();
}

template <typename T>
inline Complex<T>
exri (Complex<T> const& z)
{
  return Complex<T> (im(z), re(z));
}

template <typename T>
inline Complex<T>
operator ~ (Complex<T> const& z)
{
  return Complex<T> (re(z), -im(z));
}

template <typename T>
inline Complex<T>
conj (Complex<T> const& z)
{
  return Complex<T> (re(z), -im(z));
}

template <typename T>
inline T
abs (Complex<T> const& z)
{
  return T (hypot(re(z), im(z)));
}

template <typename T>
inline T
norm (Complex<T> const& z)
{
  return re(z)*re(z) + im(z)*im(z);
}

template <typename T>
inline T
arg (Complex<T> const& z)
{
  return T (atan2(im(z),re(z)));
}

template <typename T>
inline Complex<T>
rect (Complex<T> const& z)
{
  return Complex<T> (re(z)*cos(im(z)), re(z)*sin(im(z)));
}

template <typename T>
inline Complex<T>
rect (T const r, T const t) 
{
    return Complex<T> (r*cos(t), r*sin(t));
}

template <typename T>
inline Complex<T>
polar (Complex<T> const& z) 
{
  return Complex<T> (abs(z), arg(z));
}

template <typename T>
inline Complex<T>
polar (T const x, T const y) 
{
  return Complex<T> (hypot(x,y), atan2(y,x));
}

#ifdef HAVE_NAMESPACE
}
#endif

#endif // SL_COMPLEX_CMISC_H
