/*
 ******************************
 * Scientific Library (GNU Public Licence)
 *
 * Author: Laurent Deniau, Laurent.Deniau@cern.ch
 *
 * $Id: complex_test.cc,v 1.11 1998/08/14 09:00:36 paubert Exp $
 *
 * Suggestions: sl@mathinsa.insa-lyon.fr
 * Bugs:   sl-bugs@mathinsa.insa-lyon.fr
 *
 * For more information, please see the sl++ Home Page:
 * http://wwwinfo.cern.ch/~ldeniau/sl.html
 *
 ******************************
 */

#include "defs.h"

#ifdef HAVE_IOSTREAM
#include <iostream>
#else
#  if HAVE_IOSTREAM_H
#    include <iostream.h>
#  else
#    error cannot find <iostream> or <iostram.h>
#  endif
#endif

#include "complex.h"

NAMESPACE_USE(std)
NAMESPACE_USE(sl)

int main()
{
  Int n;
//  DReal d;
//  Complex<DReal> a, b, c;
  Real d;
  Complex<Real> a, b, c;

  // Modifies the I/O format
  ComplexFormat::set(0,0,'i');

  // Read numbers: input 3+2i
  
/*  cout << "Enter the complex a : ";
  cin >> a;
  cout << "Enter the complex b : ";
  cin >> b;
  cout << "Enter the double d : ";
  cin >> d;
  cout << "Enter the integer n : ";
  cin >> n; */
 
  // Results in the file complex_test.results use:
  a = Complex<>(-2.1,3.2);
  b = Complex<>(1.3,-1.2);
  d = 3.5;
  n = 5; 

  cout <<   "a = " << a << ", b = " << b 
       << ", d = " << d << ", n = " << n << '\n' ;

  cout << "\n*** Elementary functions ***" << '\n';

  cout << "real(a) = "  << real(a)  << '\n';
  cout << "imag(a) = "  << imag(a)  << '\n';
  cout << "exri(a) = "  << exri(a)  << '\n';
  cout << "conj(a) = "  << conj(a)  << '\n';
#ifdef HAVE_ALTERNATIVE_TOKEN
  cout << "not(a)  = "  << not(a)   << '\n';
#endif

  cout << "\n*** Unary operators ***" << '\n';

  cout << "+a  = "      << +a       << '\n';
  cout << "-a  = "      << -a       << '\n';
  cout << "!a  = "      << !a       << '\n';
  cout << "~a  = "      << ~a       << '\n';

  cout << "\n*** Binary operators ***" << '\n';

  cout << "a+b = "      << a+b      << '\n';
  cout << "a-b = "      << a-b      << '\n';
  cout << "a*b = "      << a*b      << '\n';
  cout << "a/b = "      << a/b      << '\n';

  cout << "a+d = "      << a+d      << '\n';
  cout << "a-d = "      << a-d      << '\n';
  cout << "a*d = "      << a*d      << '\n';
  cout << "a/d = "      << a/d      << '\n';

  cout << "d+b = "      << d+b      << '\n';
  cout << "d-b = "      << d-b      << '\n';
  cout << "d*b = "      << d*b      << '\n';
//  cout << "d/b = "      << d/b      << '\n';

  cout << "\n*** Assignment operators ***" << '\n';

  cout << "c=a  = "     << (c=a)    << '\n';
  cout << "c+=b = "     << (c+=b)   << '\n';
  cout << "c-=b = "     << (c-=b)   << '\n';
  cout << "c*=b = "     << (c*=b)   << '\n';
//  cout << "c/=b = "     << (c/=b)   << '\n';

  cout << "c=d = "      << (c=d)    << '\n';
  cout << "c+=d = "     << (c+=d)   << '\n';
  cout << "c-=d = "     << (c-=d)   << '\n';
  cout << "c*=d = "     << (c*=d)   << '\n';
  cout << "c/=d = "     << (c/=d)   << '\n';

  cout << "\n*** Comparison operators ***" << '\n';

  cout << "a==b = "     << (a==b)   << '\n';
  cout << "a!=b = "     << (a!=b)   << '\n';
  cout << "a&&b = "     << (a&&b)   << '\n';
  cout << "a||b = "     << (a||b)   << '\n';

  cout << "a==d = "     << (a==d)   << '\n';
  cout << "a!=d = "     << (a!=d)   << '\n';
  cout << "a&&d = "     << (a&&d)   << '\n';
  cout << "a||d = "     << (a||d)   << '\n';

  cout << "d==b = "     << (d==b)   << '\n';
  cout << "d!=b = "     << (d!=b)   << '\n';
  cout << "d&&b = "     << (d&&b)   << '\n';
  cout << "d||b = "     << (d||b)   << '\n';

  cout << "\n*** Coordinates functions ***" << '\n';

  cout << "abs(a)   = " << abs(a)   << '\n';
  cout << "arg(a)   = " << arg(a)   << '\n';
  cout << "norm(a)  = " << norm(a)  << '\n';
  cout << "polar(a) = " << polar(a) << '\n';
  cout << "polar(real(a),imag(a)) = " << polar(real(a),imag(a)) << '\n';
  cout << "rect(a)  = " << rect(a)  << '\n';
  cout << "rect(real(a),imag(a)) = " << rect(real(a),imag(a)) << '\n';

  cout << "\n*** Power functions ***" << '\n';

  cout << "sqrt(a)  = " << sqrt(a)  << '\n';
  cout << "exp(a)   = " << exp(a)   << '\n';
//  cout << "pow(a,b) = " << pow(a,b) << '\n';
  cout << "pow(d,b) = " << pow(d,b) << '\n';
  cout << "pow(a,d) = " << pow(a,d) << '\n';
//  cout << "pow(a,n) = " << pow(a,n) << '\n';
  cout << "ln(a)    = " << ln(a)    << '\n';
  cout << "log(a)   = " << log(a)   << '\n';
  cout << "log10(a) = " << log10(a) << '\n';

  cout << "\n*** Trigonometric functions ***" << '\n';

  cout << "sin(a)   = " << sin(a)   << '\n';
  cout << "cos(a)   = " << cos(a)   << '\n';
//  cout << "tan(a)   = " << tan(a)   << '\n';
  cout << "sinh(a)  = " << sinh(a)  << '\n';
  cout << "cosh(a)  = " << cosh(a)  << '\n';
//  cout << "tanh(a)  = " << tanh(a)  << '\n';
  cout << "asin(a)  = " << asin(a)  << '\n';
  cout << "acos(a)  = " << acos(a)  << '\n';
//  cout << "atan(a)  = " << atan(a)  << '\n';
  cout << "asinh(a) = " << asinh(a) << '\n';
  cout << "acosh(a) = " << acosh(a) << '\n';
//  cout << "atanh(a) = " << atanh(a) << endl;

  return 0;
  
}
