/*                                                            -*- c++ -*-
 ******************************
 * Scientific Library (GNU Public Licence)
 *
 * Author: Laurent Deniau, Laurent.Deniau@cern.ch
 *
 * $Id: defs.h,v 1.15 1998/11/16 17:22:12 paubert Exp $
 *
 * Suggestions: sl@mathinsa.insa-lyon.fr
 * Bugs:   sl-bugs@mathinsa.insa-lyon.fr
 *
 * For more information, please see the sl++ Home Page:
 * http://wwwinfo.cern.ch/~ldeniau/sl.html
 *
 ******************************
 */

#ifndef SL_DEFS_H
#define SL_DEFS_H

#include "sl-config.h"

// ----------------------------------------------------------------------
// Deal with includes
// ----------------------------------------------------------------------
#ifdef HAVE_IOSTREAM
#  include <iostream>
#else
#  ifdef HAVE_IOSTREAM_H
#    include <iostream.h>
#  else
#    error Cannot found <iostream.h> or <iostream>
#  endif
#endif

#ifdef HAVE_FSTREAM
#  include <fstream>
#else
#  ifdef HAVE_FSTREAM_H
#    include <fstream.h>
#  endif
#endif

#ifdef HAVE_ISTREAM
#  include <istream>
#else
#  ifdef HAVE_ISTREAM_H
#    include <istream.h>
#  endif
#endif

#ifdef HAVE_CSTDDEF
#  include <cstddef>
#else
#  ifdef HAVE_STDDEF_H
#    include <stddef.h>
#  else
#    error Cannot found <cstddef> or <stddef.h>
#  endif
#endif

#ifdef HAVE_CASSERT
#  include <cassert>
#else
#  ifdef HAVE_ASSERT_H
#    include <assert.h>
#  else
#    error Cannot find <cassert> or <assert.h>
#  endif
#endif

#ifdef HAVE_CLIMITS
#  include <climits>
#else
#  ifdef HAVE_LIMITS_H
#    include <limits.h>
#  else
#    error Cannot find <climits> or <limits.h>
#  endif
#endif

#ifdef HAVE_CMATH
#  include <cmath>
#else
#  ifdef HAVE_MATH_H
#    include <math.h>
#  else
#    error Cannot find <cmath> or <math.h>
#  endif
#endif

#ifdef HAVE_ALGO
#  include <algo>
#else
#  ifdef HAVE_ALGO_H
#    include <algo.h>
#  endif
#endif

#ifdef HAVE_ALGORITHM
#  include <algorithm>
#else
#  ifdef HAVE_ALGORITHM_H
#    include <algorithm.h>
#  endif
#endif

#ifdef HAVE_CFLOAT
#  include <cfloat>
#else
#  ifdef HAVE_FLOAT_H
#    include "float.h"
#  else
#    error Cannot find <cfloat> or <float.h>
#  endif
#endif

// Needs definition of DBL_EPSILON
#if ! ((defined HAVE_FLOAT_H) && (defined HAVE_CFLOAT))
#  ifndef DBL_EPSILON
#    define DBL_EPSILON 2.2204460492503131e-16
#  endif
#  ifndef FLT_EPSILON
#    define FLT_EPSILON ((float)1.19209290E-07)
#  endif
#endif

// I suppose that the value of $\pi$ is constant!
#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

#ifdef HAVE_NAMESPACE
namespace sl
{
#endif    

// Definition of sl++ types

#ifndef SL_BOOL_TYPE
#  define SL_BOOL_TYPE bool
#endif

#ifndef SL_CHAR_TYPE
#  define SL_CHAR_TYPE char
#endif

#ifndef SL_UCHAR_TYPE
#  define SL_UCHAR_TYPE unsigned char
#endif

#ifndef SL_SHORT_TYPE
#  define SL_SHORT_TYPE short int
#endif

#ifndef SL_USHORT_TYPE
#  define SL_USHORT_TYPE unsigned short int
#endif

#ifndef SL_INT_TYPE
#  define SL_INT_TYPE int
#endif

#ifndef SL_UINT_TYPE
#  define SL_UINT_TYPE unsigned int
#endif

#ifndef SL_LONG_TYPE
#  define SL_LONG_TYPE long int
#endif

#ifndef SL_ULONG_TYPE
#  define SL_ULONG_TYPE unsigned long int
#endif

#if defined(HAVE_LONG_LONG)
#  ifndef SL_DLONG_TYPE
#    define SL_DLONG_TYPE long long int
#  endif
#  ifndef SL_UDLONG_TYPE
#    define SL_UDLONG_TYPE unsigned long long int
#  endif
#else
#  ifndef SL_DLONG_TYPE
#    define SL_DLONG_TYPE long int
#  endif
#  ifndef SL_UDLONG_TYPE
#    define SL_UDLONG_TYPE unsigned long int
#  endif
#endif

#ifndef SL_SREAL_TYPE
#  define SL_SREAL_TYPE float
#endif

#ifndef SL_REAL_TYPE
#  define SL_REAL_TYPE double
#endif

#ifndef SL_DREAL_TYPE
#  define SL_DREAL_TYPE long double
#endif

#ifndef SL_STRING_TYPE
#  define SL_STRING_TYPE char*
#endif

#ifndef SL_INDEX_TYPE
#  define SL_INDEX_TYPE size_t
#  define SL_INDEX_MIN  1
#  define SL_INDEX_MAX  size_t(-1)
#endif

typedef SL_BOOL_TYPE   Bool;
typedef SL_CHAR_TYPE   Char;
typedef SL_UCHAR_TYPE  UChar;
typedef SL_SHORT_TYPE  Short;
typedef SL_USHORT_TYPE UShort;
typedef SL_INT_TYPE    Int;
typedef SL_UINT_TYPE   UInt;
typedef SL_LONG_TYPE   Long;
typedef SL_ULONG_TYPE  ULong;
typedef SL_DLONG_TYPE  DLong;
typedef SL_UDLONG_TYPE UDLong;
typedef SL_SREAL_TYPE  SReal;
typedef SL_REAL_TYPE   Real;
typedef SL_DREAL_TYPE  DReal;

typedef SL_STRING_TYPE String;
typedef SL_INDEX_TYPE  Index;




Index const MinIndex = SL_INDEX_MIN;
Index const MaxIndex = SL_INDEX_MAX;

#ifdef HAVE_NAMESPACE
}
#endif 

// don't include the following as they are not needed to compile traits_generator
// the build can fail if they are included
#ifndef IN_TRAITS_GENERATOR
#include "traits.h"
#include "utils.h"
#include "error.h"
#endif


#endif
// #ifndef SL_DEFS_H
