/*                                                           -*- c++ -*-
 ***********************************************************************
 * Scientific Library (GNU Public Licence)
 *
 * Exception handling
 *
 * Author: Pierre Aubert paubert@mathinsa.insa-lyon.fr
 *
 * $Id: error.h,v 1.4 1998/07/16 15:02:13 paubert Exp $
 *
 * Suggestions: sl@mathinsa.insa-lyon.fr
 * Bugs:   sl-bugs@mathinsa.insa-lyon.fr 
 *
 * For more information, please see the sl++ Home Page:
 * http://wwwinfo.cern.ch/~ldeniau/sl.html 
 *
 ***********************************************************************
 */

#ifndef SL_ERROR_H
#define SL_ERROR_H

#ifdef HAVE_NAMESPACE
namespace sl 
{
#endif

  /// list of all #ErrorCode# 
  enum ErrorCode
  {
    /// default cases
    eNoError,
    /// an error of unidentified type occured
    eError,
    /// not yet implemented
    eNotYetImplemented,
    /// code is known to be bogus
    eCodeIsBogus,
    /// the virtual member has not been implemented, you use the default one
    eVirtualNotImplemented,
    /// the call to constructor has used wrong parameter(s)
    eWrongConstructorCalled,
    /// dimensions is negative
    eDimensionNegative,
    /// dimensions are not compatible
    eDimensionIncompatible,
    /// not egough memory 
    eNotEnoughMemory,
    /// matrix is not positive definite
    eMatrixIsNotPositiveDefinite,
    /// matrix is must be square 
    eMatrixMustBeSquare,
    /// allocation of memory failed 
    eMemoryAllocationFailed,
    /// must be positive
    eMustBePositive,
    /// division by zero
    eDivisionByZero,
    /// illegal operation
    eIllegalOperation,
    /// meaningless operation 
    eMeaningLess,
    /// other error
    eOtherError
  };


#ifdef HAVE_NAMESPACE
}
#endif



#endif
// end of SL_ERROR_H definition(s)
// do not write anything after this line!

