#include "newt.h" 

void newt(PathFollow &P, StepCtrl &S, int *corrIteration, double t, Complex<Real> x[], Complex<Real> hValCurrent[],int *statusC, double tauRef, double powerRef,
double *rowScaling, double *colScaling,
        void (*vecfunc)(PathFollow &, double, Complex<Real> [], Complex<Real> [], double [],double, double))

/* Input 

*/
/* Given an initial guess x[1..n] for a root in n dimensions, find the root 
   by a globally convergent Newton's method. The vector of functions to be 
   zeroed, called hValCurrent[1..n] in the routine below, is returned by the 
   user-supplied routine vecfunc(n,x,hValCurrent). The output quantity check is 
   false (0) on a normal return and true (1) if the routine has converged 
   to a local minimum of the function fmin defined below. In this case try 
   restarting from a different initial guess. */
{
double contr,contr2;
int i,its,j,n,id,d; 
double dist,dist1,distOld,NewtonDirMax,distMin,
	distThreshold,hValError,normOFx,sum,temp,test,hValErr2;
Complex<Real> ctest,csum;

n=P.xDim;

nn=n; 
funcv=vecfunc; 

normOFx=DMAX(infNorm(x,n),1.0);
NewtonDirMax=S.NewtonDirMax*normOFx ;
distMin= DMIN(S.distMin*normOFx,0.001);
distThreshold=S.distThreshold*normOFx;
*statusC=0;   //2: success & expand predStep; 1: succeess & maintain predStep;
		// 0: continue; -1: fail & retry with halved predStep
*corrIteration=0;
contr=0.0;
contr2=0.0;
hValError=infNorm(hValCurrent,n);	//hValCurrent is also computed by this call. 

if(P.Verbose >=1) cout<<" newton it  0 "<<" norm of f "<<hValError<<endl;
if(hValError < P.hValMin ) {*statusC=2;FREERETURN;} // if t=0, return right away.

fdjac(P,t,x,rowScaling,colScaling,tauRef, powerRef); 


if(P.Verbose >=1) 
  cout<<" newton it "<< *corrIteration <<" norm of f "<<hValError<<endl;
if(P.Verbose >=3) { cout<<"Hjac in Newton "<<endl;
	for(i=1;i<=n;i++) {
	    for(j=1;j<=n;j++) 
		cout<<" "<<P.Hjac[i][j];cout<<endl; }cout<<endl; }

for (i=1;i<=n;i++) S.p[i] = -hValCurrent[i]; 	// Right-hand side for linear
// debugging
//	for (i=1;i<=n;i++) for(j=1;j<=n;j++) Hjac[i][j]=P.Hjac[i][j];
// 	cout<<" -hValCurrent "<<endl;for(i=1;i<=n;i++)cout<< S.p[i]<<" "; cout<<endl;
//qrdcmp(P.Hjac,n,P.cc,P.dd,&id); 
hValErr2=twoNorm(hValCurrent,n);

c2r2(P.Hjac,n,P.GMat,hValCurrent,P.HtR); // convert to R^2n space & RHS
svdcmp(P.GMat,2*n,2*n,P.svdW,P.svdV);
for (i=1; i<=2*n; i++)  {       // small perturbation for nonsingularity of W
        P.svdW[i] = DMAX(P.svdW[i],DMIN(P.extSmallNumber,hValErr2));
        P.HtR[i] = -P.HtR[i];           //take negative sign for Newton eq.
}
svbksb(P.GMat,P.svdW,P.svdV,2*n,2*n,P.HtR,P.ccR);
r2c(P.HtR,n,S.p);                        // convert back to complex

if(P.Verbose>=1) {
	ctest=(1,1); for (i=1; i<=n; i++) {ctest = ctest*P.dd[i];
	if(P.Verbose>=3) cout<<" D "<<P.dd[i]<<endl; }
	cout<<"norm J "<<ctest<<endl; 
}
//qrsolv(P.Hjac,n,P.cc,P.dd,S.p); 
if (P.scalingSW ==2) {
	for (i=1;i<=n;i++) S.p[i] /=colScaling[i];
}

for (dist=0.0,i=1;i<=n;i++) dist += norm(S.p[i]);  // 2-norm
dist = sqrt(dist);	// compute norm of dx

while (*statusC==0) {
	*corrIteration +=1;

	if(*corrIteration==1) {
		dist1 =dist; distOld = dist;
		//cout<<" dist1 "<<dist1<<" "<<NewtonDirMax<<endl;
		if (dist1 > NewtonDirMax) { 
			*statusC=-1; FREERETURN }
	} else {
		contr = dist/distOld; 
		if (*corrIteration==2 ) contr2=contr;
		if (NewtonDirMax < dist) { *statusC= -1; FREERETURN }
		else if (S.ctMax <contr ) { 
			*statusC=-2; FREERETURN }
		distOld = dist ;
	}
	if (*statusC==0) {
		for (j=1;j<=n;j++)
			x[j] +=S.p[j];  
		// computer hValCurrent and Jacobian
		hValError=fmin(P,t,x,hValCurrent,rowScaling,tauRef,powerRef);	
		if(P.Verbose >=1) 
                   cout<<" newton it "
			<< *corrIteration <<" norm of f "<<hValError<<endl;
		fdjac(P,t,x,rowScaling,colScaling,tauRef, powerRef); 
		//qrdcmp(P.Hjac,n,P.cc,P.dd,&id); 
		//for (i=1;i<=n;i++) S.p[i] = -hValCurrent[i]; // Right-hand side for linear
 		//qrsolv(P.Hjac,n,P.cc,P.dd,S.p); 
		c2r2(P.Hjac,n,P.GMat,hValCurrent,P.HtR); // to R^2n space & RHS
		svdcmp(P.GMat,2*n,2*n,P.svdW,P.svdV);

		for (i=1; i<=2*n; i++)  {// small perturbation for nonsingularity of W
		        P.svdW[i] = DMAX(P.svdW[i],DMIN(P.extSmallNumber,hValErr2));
        		P.HtR[i] = -P.HtR[i]; //take negative sign for Newton eq.
}
		svbksb(P.GMat,P.svdW,P.svdV,2*n,2*n,P.HtR,P.ccR);
		r2c(P.HtR,n,S.p);                        // convert back to complex
		if (P.scalingSW ==2) {
			for (i=1;i<=n;i++) S.p[i] /=colScaling[i];
		}
		for (dist=0.0,i=1;i<=n;i++) dist += norm(S.p[i]);  // 2-norm
		dist = sqrt(dist);	// compute norm of dx

		if (dist<S.distMin || hValError<P.hValMin ) {
			if(dist1<distThreshold && contr2<S.ctThreshold)
				*statusC=2;
			else
				*statusC=1;
				
			FREERETURN
		} else if (*corrIteration>=P.corrItMax) {
			*statusC=-3;
			FREERETURN	
		}
		
	}
}	
}
//===========================================================================
#include "strucPATH.h" 
#define EPS 1.0e-25 	/* Approximate square root of the machine precision. */
void indexx(int n, double arr[], int indx[], int istack[]);

void fdjac(PathFollow  &P, double t,Complex<Real> x[],double rowScaling[], double colScaling[],double tauRef, double powerRef)

/* 
   On input, x[1..n] is the point at which the Jacobian is to be 
   evaluated, hValCurrent[1..n] is the vector of function values at the point, 
   and vecfunc(n,x,f) is a user-supplied routine that returns the vector 
   of functions at x. On output, Hjac[1..n][1..n] is the Jacobian array. */
{

int i,j,k,l,m,mm,n,NoColScaling;
double d,tpower; 
Complex<Real> oneTerm;

n=P.xDim;
// if the number of terms is not changed for combined homotopy, adjust P.Coef[i][j]
// to include generic and original coefficients with t. (6/17/01)

// 10/29/01 changes to include beta --------------------------------->
tpower = pow(t, P.beta/P.gammaP);
for  (i=1;i<=n;i++)  {
     for (j=1;j<=P.numTerm[i];j++)
          P.Coef[i][j]=P.CoefSav[i][j]+(P.Coef_ori[i][j]-P.CoefSav[i][j])*tpower;
}
//<-----------------------------------------------------------
for (i=1; i <= n; i++){                 // i for equation
    for (j=1; j <= n; j++) {        // j for diff w.r.t.the variable
        P.Hjac[i][j]= Complex<>(0.0,0.0);
  // to have tauRef, powerRef used
            for ( l=1; l <= P.numTerm[i]; l++) {
                if (  P.Power[i][l][j] != 0.0) {
                    oneTerm =  P.Coef[i][l]* P.Power[i][l][j]*
                                //(pow(t,P.powerT[i][l]))*
                                (pow(tauRef,(P.powerT[i][l])/powerRef))*
                                ( pow(x[j],(P.Power[i][l][j]-1.0) ));
                    for (m=1; m <=n; m++)
                        if(m !=j)  oneTerm *=pow(x[m],P.Power[i][l][m]);
                    P.Hjac[i][j] += oneTerm;
                }
            }

    	}
    }
//----------------------Scaling-----------------------------------------
NoColScaling=0; 		// to prevent dividing by 0.
if (P.scalingSW >=1 ) {
        for (i=1;i<=n;i++) {
                for (j=1;j<=n;j++)
                        P.Hjac[i][j] /= rowScaling[i];
        }
        if (P.scalingSW ==2 ) {
            for (i=1;i<=n;i++) {
                colScaling[i] =P.Hjac[1][i]; // initialize with first ele
                for (j =2;j<=n;j++)
                    colScaling[i]= DMAX(colScaling[i],abs(P.Hjac[j][i]));
		if(colScaling[i]==0.0) NoColScaling=1;
            }
	    if(NoColScaling==0) { 
                for (i=1;i<=n;i++) {
                    for (j=1;j<=n;j++)
                        P.Hjac[i][j] /= colScaling[j];
                }
	    }

        }
}


}

//=============================================================================

extern int nn; 
extern void (*funcv)(PathFollow  &P, double t, Complex<Real> x[], Complex<Real> f[], double rowScaling [],double tauRef, double powerRef);

double fmin(PathFollow  &P, double t, Complex<Real> x[],Complex<Real> hValCurrent [],double rowScaling [], double tauRef, double powerRef ) 
/* The global pointer *funcv points 
   to a routine that returns the vector of functions at x. It is set to 
   point to a user-supplied routine in the calling program. Global 
   variables also communicate the function values back to the calling 
   program. */
{ 

int i; 
double sum;

(*funcv)(P,t,x,hValCurrent,rowScaling,tauRef,powerRef); 
//for (sum=0.0,i=1;i<=nn;i++) sum += norm(hValCurrent[i]);  // 2-norm
//return 0.5*sum;
for (sum=0.0,i=1;i<=nn;i++) {
	if( norm(hValCurrent[i]) >=sum) sum=norm(hValCurrent[i]);  // infinity-norm
	}
return sqrt(sum);
}  
