#include <math.h>
#include "nrutil.h"
#include "strucPATH.h"

void qrdcmp(Complex<Real> **a, int n, Complex<Real> *c, Complex<Real> *d, int *sing)
{
/* Construct the QR decomposition of a[1..n][1..n]. The upper triangular
   matrix R is returned in the upper triangle of a, except for the 
   diagonal elements of R which are returned in d[1..n]. The orthogonal
   matrix Q is represented as a product of n-1 Householder matrices
   Q_1...Q_{n-1} where Q_j=1-u_ju_j^T/c_j. The i-th component of u_j
   is zero for i=1,...,j-1 while the nonzero components are returned
   in a[i][j] for i=j,...n. sing returnes as true (1) if singularity is
   encountered during the decomposition, but the decomposition is still
   completed in this case; otherwise it returns false (0) */

	int i,j,k;
	double scale=0.0,sum;
	Complex<Real> csum,sigma,tau;

//for(i=1;i<=n;i++) {cout<<"a in qrd "<<endl;{for(j=1;j<=n;j++) cout<<a[i][j]<<" ";}cout<<endl;}
	*sing=0;
	for (k=1;k<n;k++) {
		for (i=k;i<=n;i++) scale=DMAX(scale,abs(a[i][k]));
		if (scale == 0.0) { 		// singular case
			*sing=1;
			c[k]=d[k]=Complex<>(0.0,0.0);
		} else {			// Form Q_k and Q_k A
			for (i=k;i<=n;i++) a[i][k] = a[i][k]/scale;
			for (sum=0.0,i=k;i<=n;i++) sum +=norm(a[i][k]);

//			sigma=SIGN(sqrt(sum),(a[k][k]));
//	take real part to avoid too samll number when added to a[k][k]
//			tau=real(a[k][k])+imag(a[k][k]);
			sigma=SIGN(sqrt(sum),real(a[k][k]));
			a[k][k] += sigma;
			c[k]=sigma*a[k][k];
			d[k] = -scale*sigma;
			for (j=k+1;j<=n;j++) {
				for (csum=Complex<>(0.0,0.0),i=k;i<=n;i++) csum += conj(a[i][k])*(a[i][j]);
				tau=csum/(c[k]);
				for (i=k;i<=n;i++) a[i][j] -= tau*(a[i][k]);
			}
		}
	}
	d[n]=a[n][n];
	if (abs(d[n]) == 0.0) *sing=1;
//for(i=1;i<=n;i++) {cout<<"a in qrd "<<endl;{for(j=1;j<=n;j++) cout<<a[i][j]<<"  ";}cout<<endl;}
//for(i=1;i<=n;i++) {cout<<"d in qrd "<<d[i]<<" ";}cout<<endl;
//for(i=1;i<=n;i++) {cout<<"c in qrd "<<c[i]<<" ";}cout<<endl;
}
