#ifndef STRUCPATH_H  
#define STRUCPATH_H  

#include "defs.h"
#include "complex.h"
#include <fstream.h>
#include <iostream.h>
#include <math.h>
#include <new.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/times.h>
#include <time.h>

#define NSTACK 250 		// constant for indexx.c    
#define Debug 0
#define TIMES_PER_SECOND   CLK_TCK
#define Eps 1.0e-7
#define SPARSESW 0
#define CHANGEDIM 1
static double dmaxarg1, dmaxarg2;
#define DMAX(a,b) (dmaxarg1=(a),dmaxarg2=(b),(dmaxarg1) > (dmaxarg2) ? (dmaxarg1) : (dmaxarg2))
static double dminarg1, dminarg2;
#define DMIN(a,b) (dminarg1=(a),dminarg2=(b),(dminarg1) < (dminarg2) ? (dminarg1) : (dminarg2))

NAMESPACE_USE(std)
NAMESPACE_USE(sl)

class PathFollow {                            
// PathFollow defines a system of equations and its Jacobian
public:
	double	 ***Power;
	Complex<Real>	 **Coef,**CoefSav; 	// CoefSav for generic coefficients
	double	 **Coef_ori;
	double	 **powerT;
	double	 **tmpowerT;			// to compute jac, fun
	double	 **linpowerT;			// to save t in linear homotopy 
	Complex<Real> 	 **Sol;
	Complex<Real> 	 **Hjac;		// Jacobian
	Complex<Real> 	 *term;			// stroge for computing jacobian
	Complex<Real> 	 *cc;			// stroge for computing jacobian
	Complex<Real> 	 *dd;			// stroge for computing jacobian
	Complex<Real> 	 *v,*dv;		// stroge for euler step
        int      *numTerm;			//for each equation
	int 	 **sindx, *tindx;		// index for sorted powerT
        int      numSol;
        int      xDim;                   	// dimension
	int 	*istack;	 		// in indexx to compute jacboain
	int	corrItMax,Verbose;		// newton method parameter 
	double	hValMin,TOLMIN,normOFdx,normOFx;// newton method parameters
	double 	maxPower;			// max t power
	double	beta;			// array for high t value,scale
	double 	*logABSx,*ANGLEx,*logMagOfTerms,*AngleOfTerms;
	Complex<Real> *terms;
	double  **svdV,*svdW,**GMat;		// storage for computing condi #
	Complex<Real>  **csvdV,*csvdW;		// storage for csvdcmp
	double  condNo,det,minEigForNonsing,minEig,maxEig,smallestEig,gammaP;	
	int	scalingSW,inputFormat,modSW,singularSW;
	double  *ccR,*HtR;		// for c2r, r2c
	double 	extSmallNumber; 

	PathFollow(void);
	~PathFollow();
	void   StoreVec(int i, int j, int k, int val){ Power[i][j][k]=val; }
	void   StoreCoef(int j, int k, Complex<Real> val){ Coef[j][k] = val; }
	void   StoreCoef_ori(int j, int k, double val){ Coef_ori[j][k] = val; }
	void   StoreT(int j, int k, double val){ powerT[j][k] = val; }
	double Vec(int i, int j, int k){  return( Power[i][j][k] ); }
	void   StoreSol(int j, int k, Complex<Real> val){ Sol[j][k] = val; }
        void   Allocate(int nCon, int	*index);
        void   Allocate4Sol(int nCon, int Nsol);
};  
class StepCtrl {
public:
	int 	xDim,predIteration,totalCorrIteration;
        int     *numTerm,*istack,totalNoTerms;	//for each and all equations

	int	*tVInd,informationLevel;
	double	*tPowerVector;  // step control based on tmaxpw
	
	double	neglSmallNumber,pStepMax,pStepMax2; // step control from matlab
	double 	angleMin,angleThUp,angleThDown,predItMax,pStepMin,
		predStep0,expansionFactor,reductionFactor,dTauMin,dTauPredOrder,
		dTauMax,dTau0,dTau, divValueOFt,divMagOFx,divNormOFdx,divRatio,
		distMin,NewtonDirMax,distThreshold,tMax,tScalingFactor,
		ctThreshold,ctMax,minPredStep,normOFdx,
		cITthreshold, tauValIntoNewton, tValIntoNewton, accINfVal,
		accInNewtonDir,FinaldistMax,FinalCorrItMax,Final2hvalMin, 
		Final2distMin,tEqualTo1,tauEqualTo1;
	Complex<Real> *hValOld,*hValPredicted,*pDirect0,*pDirect1,*pDirect2,
		*secondOrderTerm,*xtPredicted,*xOld,*xtTarget,*hValTarget,
		*xtVect0,*xtNew,*hValNew,*p;
	double	*rowScaling,*colScaling;
 
	StepCtrl(void);
	~StepCtrl();
        void    Allocate(int nCon,int *index);
        void    AllocateTpowerV(int ttalNoTerms);
}; 
class Binomials {
public:
        int     numQ;                   	// how many Q(x) 
        int     *nSOLQ;                		// how many sols in each Q(x)
        int     xDim;                		// dimension     
        int     intervalT;                   	// # of interval for high p t
	StepCtrl	Step;			// step control
	PathFollow	*Qx;			// binomials
	Complex<Real>	*xini,*sav_x,*Hval; 	// used in predictor/corre
	double	*solution;			// used for print sol
	int	nstep;				// # of steps in euler,tt size 
        char	InputFileName[100],
		InputFileNamePa[100],		// parameter file
		InputFileCell[100],		
		OutputFileName[100],
	        OutputFileStat[100],
	        OutputFileNote[100],
	        OutputHeadName[100],
	        ClusterName[100],
	        SolBuffer[16384],
		StatBuffer[16384];
	Binomials(void);
	~Binomials();
	void	Allocate(int nQ, int *inNterm, int xD);
	void	Allocate4Sol(int nQ, int Nsol, int xD);
};	
	
class PrintInfo  {                        		// print information 
public:
        int    nIteration, dim;
	void   PrintStat(Binomials &Problem, float time,int i, char *cell,
			double infN, int statP, int *previousPrint);
	void   PrintStatNote(Binomials &Problem, float time,int i, char *cell,
			double infN, int statP);
	void   PrintSol(Binomials &Problem, int statP, int i, char *cell, 
			double hValError);
	void   Headings(char *outputFID, Binomials &B);
	void   ExpParam(char *outputFID, int informationLevel);
	void   Parameters(char *FID, double accINfVal, double accInNewtDir, double beta,
        double divMagOFx, double dTauMax, double minEigForNonsing, double NewtonDirMax,
        double predItMax, int informationLevel);
	void   HeaderName(char *OutputFileNote,char *OutputHeadName, int solno, int numSol);
	void   HeaderInfo(char *OutputFileNote);
	void   MidPrint(char *OutputFileNote, int predIteration, int statusC,
                double tCurrent, double tToMaxPower, double hValError,
                double normOFx, double predStep, int corrIteration, double theta, double minEig);
        void   FinalSolPrint(Binomials& Problem, float time);
	PrintInfo(void);
	~PrintInfo();
}; 
#endif
