/*                                                       -*- c++ -*-
 ******************************
 * Scientific Library (GNU Public Licence)
 *
 * Author: Laurent Deniau, Laurent.Deniau@cern.ch
 *
 * $Id: utils.h,v 1.10 1998/11/16 17:22:12 paubert Exp $
 *
 * Suggestions: sl@mathinsa.insa-lyon.fr
 * Bugs:   sl-bugs@mathinsa.insa-lyon.fr
 *
 * For more information, please see the sl++ Home Page:
 * http://wwwinfo.cern.ch/~ldeniau/sl.html
 *
 ******************************
 */

#ifndef SL_UTILS_H
#define SL_UTILS_H

#ifdef HAVE_NAMESPACE
namespace sl
{
#endif    

// Cast checking for safe conversion

template <typename T1, typename T2>
inline void
check_cast ()
{
  if ( sizeof(T1) < sizeof(T2) ) {
    assert (!"invalid type cast T1(T2): sizeof(T1)<sizeof(T2)");
  }
}

template <typename T1, typename T2>
inline T1
checked_cast (T2 x)
{
  check_cast<T1,T2>();
  return T1(x);
}

// Square function

template <typename T>
inline T
sqr (T a)
{
  return a*a;
}

// Power function, a^n

template <typename T>
inline T
pow (T a, Int n)
{
  T r(1);

  if (n < 0) { n = -n; a = T(1)/a; }
  for (;;) {
    if (n & 1)   r *= a;
    if (n >>= 1) a *= a;
    else return r;
  }
}

// Swap function
#ifndef HAVE_SWAP_IN_STL
template <typename T>
inline void
swap (T& a, T& b)
{
  T t = a;
  a = b;
  b = t;
}
#endif

// Sign function

template <typename T>
inline Int
sign (T a)
{
  return a < 0 ? -1 : 1;
}

// Min Max functions
#ifndef HAVE_MIN_IN_STL
template <typename T>
inline T
min (T a, T b)
{
  return a < b ? a : b;
}
#endif

#ifndef HAVE_MAX_IN_STL
template <typename T>
inline T
max (T a, T b)
{
  return a > b ? a : b;
}
#endif
// Reference functions

template <typename T>
T&
zero_ref()
{
  static T _z;
  
  return _z;
}

template <typename T>
T const&
zero_const_ref()
{
  static T const _z=T();
  
  return _z;
}

template <typename T>
T&
one_ref()
{
  static T _z;
  
  return _z;
}

template <typename T>
T const&
one_const_ref()
{
  static T const _z=T(1);
  
  return _z;
}

/*
 * -------------
 * Handle String
 * -------------
 */

#define _MkString_(a) #a
#define _String_(a) _MkString_(a)

/*
 * -----------------
 * Handle Built type
 * -----------------
 */

#define _MkConcat_(a,b) a ## b
#define _Concat_(a,b) _MkConcat_(a,b)

#ifdef HAVE_NAMESPACE
}
#endif

#endif
// #ifndef SL_UTILS_H
